/*
 *  hash.c
 *  NGSnexalign
 *
 *  Created by Timo Lassmann on 30/04/2008.
 *  Copyright 2008 RIKEN Yokohama Institute, Genome Exploration Research Group. All rights reserved.
 *
 */

#include "ngs_toolkit.h"

static int r[256];

void insert_hash(struct query_table* qt,struct parameters* param, char *text,int len, double count)
{
	struct query* p = 0;
	struct query* q = 0;
	struct query* list = 0;
	static int string_pos = 0;
	static int park_pos = 0;
	int hash_key = 0;
	int i;
	
	if(string_pos + len >= (param->query_num_limit*30) || park_pos >= param->query_num_limit){
		//print_hash(qt);
		//fprintf(stderr,"%d	%d	%f\n",qt->numseq,qt->size,qt->total_expression);
		fprintf(stderr,"\nWARNING cannot unique: try running nexalign with -hash_size X where X >= your number of input sequences.\n\n");
		check_sequences(qt,param);
		if(param->solid){
			mapping_solid(qt, param);
		}else{
			mapping(qt,param);
		}
		if(param->outfile){
			fclose(foutfile);
			if(param->fast || param->fragment){
				copy_mapping_output(qt,param);
			}else{
				clean_mapping_output(qt,param);
			}
			if ((unlink(param->outfile_tmp)) == -1){
				perror("Removing temporary file failed");
				exit(-1);
			}
			if ((foutfile = fopen(param->outfile_tmp, "w")) == NULL){
				fprintf(stderr,"can't open output\n");
				exit(-1);
			}
		}
		
		
		clear_hash_table(qt);
		string_pos = 0;
		park_pos = 0;
	}
	//assert(string_pos + len < HASH_STORAGE);
	//assert(park_pos < PARK_STORAGE);
	
	
	int max = KEY_LEN > len ? len: KEY_LEN;
	hash_key = r[(int)text[0]];
	for(i = 1;i < max;i++){
		hash_key = hash_key << 1 ^ r[(int)text[i]];
	}
	list = qt->hash[hash_key];
	
	if(!list  || strcmp(list->seq, text) > 0) {
		//make new p;
		
		p = qt->park[park_pos];//malloc(sizeof(struct hash_elem));
		qt->numseq++;
		qt->size++;

		qt->total_expression += count;
		p->seq = qt->string + string_pos;
		p->len = len;
		p->sn_len = 0;
		p->sn = 0;
		p->count = count;
		for(i = 0;i < len;i++){
			qt->string[string_pos] = text[i];
			string_pos++;
		}
		qt->string[string_pos] = 0;
		string_pos++;
		park_pos++;
		p->next = list;
		qt->hash[hash_key] = p;
	}else if(strcmp(list->seq, text) == 0){
		list->count += count;
		qt->total_expression += count;
	} else {  
		/* search the linked list for the right location */
		q = list;
		i = 1;
		while(q->next) {
			if(strcmp(q->next->seq, text) > 0){
				break;
			}else if(strcmp(q->next->seq, text) == 0){
				q->next->count += count;
				qt->total_expression += count;
				i = 0;
				break;
			}else{//if(strcmp(q->next->seq, text) < 0)
				q = q->next;
			}
			
		}
		//make new p;
		if(i){
			//make new p;
			p = qt->park[park_pos];
			qt->numseq++;
			qt->size++;
			qt->total_expression += count;
			p->seq = qt->string + string_pos;
			p->len = len;
			p->count = count;
			p->sn_len = 0;
			p->sn = 0;
			for(i = 0;i < len;i++){
				qt->string[string_pos] = text[i];
				string_pos++;
			}
			qt->string[string_pos] = 0;
			string_pos++;
			park_pos++;
			
			
			p->next = q->next;
			q->next = p;
		}
	}
}

struct query* lookup_hash(struct query_table* qt,char *text,int len)
{
	struct query* q = 0;
	struct query* list = 0;
	int hash_key = 0;
	int i;	
	int max = KEY_LEN > len ? len: KEY_LEN;
	hash_key = r[(int)text[0]];
	for(i = 1;i < max;i++){
		hash_key = hash_key << 1 ^ r[(int)text[i]];
	}
	list = qt->hash[hash_key];
	
	if(!list  || strcmp(list->seq, text) > 0) {
		//make new p;
		return 0;
	}else if(strcmp(list->seq, text) == 0){
		return list;
	} else {  
		/* search the linked list for the right location */
		q = list;
		i = 1;
		while(q->next) {
			if(strcmp(q->next->seq, text) > 0){
				return 0;
			}else if(strcmp(q->next->seq, text) == 0){
				return q->next;
			}else {//if(strcmp(q->next->seq, text) < 0)
				q = q->next;
			}
			
		}
	}
	return 0;
}


//new name 

void insert_hash_name(struct query_table* qt,struct parameters* param, char *seq,char* name, int len, int sn_len, double count)
{
	struct query* p = 0;
	struct query* q = 0;
	struct query* list = 0;
	static int string_pos = 0;
	static int park_pos = 0;
	int hash_key = 0;
	int i;
	
	if(string_pos + len +sn_len >= (param->query_num_limit*30) || park_pos >= param->query_num_limit){
		//print_hash(qt);
		//fprintf(stderr,"%d	%d	%f\n",qt->numseq,qt->size,qt->total_expression);
		check_sequences(qt,param);
		if(param->solid){
			mapping_solid(qt, param);
		}else{
			mapping(qt,param);
		}
		
		if(param->outfile){
			fclose(foutfile);
			if(param->fast){
				copy_mapping_output(qt,param);
			}else{
				//simple_sort_hash(qt);
				clean_name_mapping_output(qt,param);
			}
			if ((unlink(param->outfile_tmp)) == -1){
				perror("Removing temporary file failed");
				exit(-1);
			}
			if ((foutfile = fopen(param->outfile_tmp, "w")) == NULL){
				fprintf(stderr,"can't open output\n");
				exit(-1);
			}
		}
		
		
		clear_hash_table(qt);
		string_pos = 0;
		park_pos = 0;
	}
	//assert(string_pos + len < HASH_STORAGE);
	//assert(park_pos < PARK_STORAGE);
	
	
	int max = KEY_LEN > sn_len ? sn_len: KEY_LEN;
	hash_key = r[(int)name[0]];
	for(i = 1;i < max;i++){
		hash_key = hash_key << 1 ^ r[(int)name[i]];
	}
	list = qt->hash[hash_key];
	
	if(!list  || strcmp(list->sn, name) > 0) {

		p = qt->park[park_pos];//malloc(sizeof(struct hash_elem));
		qt->numseq++;
		qt->size++;

		qt->total_expression += count;
		p->seq = qt->string + string_pos;
		p->len = len;
		p->sn_len = sn_len;
		p->count = count;
		for(i = 0;i < len;i++){
			qt->string[string_pos] = seq[i];
			string_pos++;
		}
		qt->string[string_pos] = 0;
		string_pos++;
		p->sn = qt->string + string_pos;
		for(i = 0;i < sn_len;i++){
			qt->string[string_pos] = name[i];
			string_pos++;
		}
		qt->string[string_pos] = 0;
		
		p->next = list;
		qt->hash[hash_key] = p;
		p = qt->park[park_pos];
		
		string_pos++;
		park_pos++;	
		
	}else if(strcmp(list->sn, name) == 0){
		if(strcmp(list->seq, seq)!= 0){
			fprintf(stderr,"\nERROR: Found 2 sequences with the same name but different sequences:\n\n>%s\n(1)%s\n(2)%s\n",name,seq,list->seq);
			exit(-1);
		}else{
			fprintf(stderr,"\nWARNING: Found duplicated sequence ('>%s').\n",name); 
		}
		list->count += count;
		qt->total_expression += count;
	}else{  
		/* search the linked list for the right location */
		q = list;
		i = 1;
		while(q->next) {
			if(strcmp(q->next->sn, name) > 0){
				break;
			}else if(strcmp(q->next->sn, name) == 0){
				if(strcmp(list->seq, seq)!= 0){
					fprintf(stderr,"\nERROR: Found 2 sequences with the same name but different sequences:\n\n>%s\n(1)%s\n(2)%s\n",name,seq,list->seq);
					exit(-1);
				}else{
					fprintf(stderr,"\nWARNING: Found duplicated sequence ('>%s').\n",name); 
				}
				q->next->count += count;
				qt->total_expression += count;
				i = 0;
				break;
			}else{//if(strcmp(q->next->seq, text) < 0)
				q = q->next;
			}
			
		}
		//make new p;
		if(i){
			//make new p;
			
			p = qt->park[park_pos];//malloc(sizeof(struct hash_elem));
			qt->numseq++;
			qt->size++;
			
			qt->total_expression += count;
			p->seq = qt->string + string_pos;
			p->len = len;
			p->sn_len = sn_len;
			p->count = count;
			for(i = 0;i < len;i++){
				qt->string[string_pos] = seq[i];
				string_pos++;
			}
			qt->string[string_pos] = 0;
			string_pos++;
			p->sn = qt->string + string_pos;
			for(i = 0;i < sn_len;i++){
				qt->string[string_pos] = name[i];
				string_pos++;
			}
			qt->string[string_pos] = 0;
			string_pos++;
			park_pos++;
			p->next = q->next;
			q->next = p;
		}
	}
}

struct query* lookup_name_hash(struct query_table* qt,char *name,int len)
{
	struct query* q = 0;
	struct query* list = 0;
	int hash_key = 0;
	int i;	
	int max = KEY_LEN > len ? len: KEY_LEN;
	hash_key = r[(int)name[0]];
	for(i = 1;i < max;i++){
		hash_key = hash_key << 1 ^ r[(int)name[i]];
	}
	
	//fprintf(stderr,"%s	%d	%d\n",name,hash_key,max);
	
	list = qt->hash[hash_key];
	
	if(!list  || strcmp(list->sn, name) > 0) {
		//make new p;
		return 0;
	}else if(strcmp(list->sn, name) == 0){
		return list;
	} else {  
		/* search the linked list for the right location */
		q = list;
		//i = 1;
		while(q->next) {
			if(strcmp(q->next->sn, name) > 0){
				return 0;
			}else if(strcmp(q->next->sn, name) == 0){
				return q->next;
			}else {//if(strcmp(q->next->seq, text) < 0)
				q = q->next;
			}
		}
	}
	return 0;
}


void print_hash(struct query_table* qt)
{
	int i;
	struct query* p = 0;	
	for(i = 0; i<  (1<< (KEY_LEN+7));i++){
		if(qt->hash[i]){
			p = qt->hash[i];
			while(p){
				//fprintf(stderr,"%d	%d\n",p->len,p->count);
				fprintf(stderr,">%s_%f // %s \n%s\n",p->seq,p->count,p->sn,p->seq);
				//fprintf(stderr,"%s(%d,%d) ",p->seq,p->count,p->len);
				p = p->next;
			}
			//fprintf(stdout,"\n");
		}
	}
	fprintf(stderr,"%d	%f\n",qt->numseq,qt->total_expression);
	for(i = 0; i < qt->size;i++){
		fprintf(stderr,"%s	%f\n",qt->park[i]->seq,qt->park[i]->count);
	}
}


void clear_hash_table(struct query_table* qt)
{
	int i;	
	for(i = 0;i < (1<< (KEY_LEN+7));i++){
		qt->hash[i] = 0;
	}
	qt->size = 0;
}


void init_r()
{
	int i;
	srand (time(NULL) );
	for(i = 0; i < 256;i++){
		r[i] = rand()/(int)(((unsigned)RAND_MAX + 1) / 256);
	}
}


void insert_hash_with_name(struct query_table* qt,struct parameters* param, char *seq,char* name, int len, int sn_len, double count)
{
	struct query* p = 0;
	static int string_pos = 0;
	static int park_pos = 0;
	int i;
	
	if(string_pos + len+sn_len >= (param->query_num_limit*30) || park_pos >= param->query_num_limit){
		//print_hash(qt);
		//fprintf(stderr,"%d	%d	%f\n",qt->numseq,qt->size,qt->total_expression);
		check_sequences(qt,param);
		if(param->solid){
			mapping_solid(qt, param);
		}else{
			mapping(qt,param);
		}
		
		fclose(foutfile);
		
		if(param->fast){
			copy_mapping_output(qt,param);
		}else{
			if(param->uniq){
				clean_mapping_output(qt,param);
			}else{
				simple_sort_hash(qt,param->quiet_flag);
				clean_name_mapping_output(qt,param);
			}
		}
		
		if ((unlink(param->outfile_tmp)) == -1){
			perror("Removing temporary file failed");
			exit(-1);
		}
		if ((foutfile = fopen(param->outfile_tmp, "w")) == NULL){
			fprintf(stderr,"can't open output\n");
			exit(-1);
		}
		
		
		clear_hash_table(qt);
		string_pos = 0;
		park_pos = 0;
	}
	p = qt->park[park_pos];//malloc(sizeof(struct hash_elem));
	qt->numseq++;
	qt->size++;

	qt->total_expression += count;
	p->seq = qt->string + string_pos;
	p->len = len;
	p->sn_len = sn_len;
	//p->sn_len = 0;
	p->count = count;
	for(i = 0;i < len;i++){
		qt->string[string_pos] = seq[i];
		string_pos++;
	}
	qt->string[string_pos] = 0;
	string_pos++;
	p->sn = qt->string + string_pos;
	for(i = 0;i < sn_len;i++){
		qt->string[string_pos] = name[i];
		string_pos++;
	}
	qt->string[string_pos] = 0;
	string_pos++;
	park_pos++;	
}

void simple_sort_hash(struct query_table* qt,int quiet)
{
	struct query* q = 0;
	char* time_s = 0;
	int i, j;
	
	if(!quiet){
		time_s = get_time(time_s);
		fprintf(stderr,"%s	Sorting\n",time_s);
	}
	
	
	for (i = 1; i < qt->size; ++i){
		q = qt->park[i];
		for (j = i; j > 0 && strcmp(qt->park[j-1]->sn,q->sn) > 0; j--){
			qt->park[j] = qt->park[j-1];
		}
		qt->park[j] = q;
	}
	if(!quiet){
		fprintf(stderr,"%s	Finished\n",time_s);
	}
}

struct query* search_hash(struct query_table* qt,struct query* q,char *seq)
{
	int m = 0;
	int l = 0;
	int h = qt->size-1;
	int c = 0;
	if (strcmp(seq,qt->park[l]->sn)< 0){
		return 0;
	}else if (strcmp(seq,qt->park[l]->sn) ==  0){
		return qt->park[l];
	}else if(strcmp(seq,qt->park[h]->sn)>  0){
		return 0;
	}else if(strcmp(seq,qt->park[h]->sn) ==0 ){
		return qt->park[h];
	}else{
		while(h-l > 1){
			m = (l+h)/2;
			c = strcmp(seq,qt->park[m]->sn);
			//fprintf(stderr,"%d	%s	%s\n",c,seq,qt->park[m]->seq);
			if (c < 0){
				h = m;
			}else if(c == 0){
				return qt->park[m];
			}else{
				l = m;
			}
		}
	}
	return 0;
}



