/*
	interface.c
	
	Released under GPL - see the 'COPYING' file   
	
	Copyright (C) 2006 Timo Lassmann <timolassmann@gmail.com>
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
	Please send bug reports, comments etc. to:
	timolassmann@gmail.com
*/

#include "ngs_toolkit.h"

#define OPT_ABORT  1            /* --abort */
#define OPT_UNMAPPED  2           
#define OPT_MAPPED  3           
#define OPT_MULTI  4       
#define OPT_SPLIT 5 
#define OPT_INDEL 6 
#define OPT_SOLID 7
#define OPT_MAX_NO_ANN 8
#define OPT_TYPES 9
#define OPT_TIME 10
#define OPT_QUERY_NUM_LIMIT 11
#define OPT_ANN 12
#define OPT_STATS 13
#define OPT_FRAGMENT 14
#define OPT_SHORT 15
#define OPT_UNIQ 16
#define OPT_MINLENCUTOFF 17
#define OPT_MAXLENCUTOFF 18
#define OPT_FAST 19
#define OPT_NUM_THREADS 20
#define OPT_CHOP_AT 21
#define OPT_EX_CHAR 22
#define OPT_SORT 23
#define OPT_CLUSTER 24
#define OPT_GFF 25
#define OPT_BED 26
#define OPT_HIERARCHICAL 27
#define OPT_COUNT 28
#define OPT_DESCRIPTION 29

struct parameters* interface(struct parameters* param,int argc,char **argv)
{
	FILE *file = 0;
	int c,i;
	int outlen = 0;
	int help = 0;
	char *format = 0;
	static char version[] = VERSION;
	static char  usage[] =", Copyright (C) 2007,2008 Timo Lassmann <timolassmann@gmail.com>\n";
	static char help_text[] = "Usage: nexalign [options] -p <query> <database files> -o <mapping>\n\n\
     Option:\n\
     Alignment options:\n\
        -e		report exact alignments\n\
        -m X		report alignments with up to <X> mismatches\n\
        -indel		report alignments with 1 Insertion/Deletion\n\
        -solid		align SoliD color-space sequences\n\
     Sequence selection options:\n\
	-min-len X	align tags longer than 'X' (default 0).\n\
	-max-len X	align tags shorter than 'X' (default 500).\n\
	-chop-at X	aligns the first 'X' positions of tags (default 500).\n\
	-frag X		fragment input sequences into X mers\n\
        \n\
     Output options:\n\
        -o FILE		write mapping file to FILE \n\
	-format		mapping format [nexalign,blast8,blast9]\n\
        -u X		report alignments for tags mapping less than 'X' times\n\
        -unmapped FILE	write unmapped sequences to FILE\n\
        -mapped FILE	write mapped sequences to FILE\n\
        -multi FILE	write multi-mapped sequences to FILE\n\
        -stats FILE	print mapping statistics to FILE\n\
	-count FILE	write alignment statistics for each tag to FILE\n\
	-split FILE	write alignment statistics for each tag and database to FILE\n\
        -ann FILE	print annotation summary to FILE\n\
	-max_no_ann X	specify the number of overlapping annotations allowed.\n\
        -types FILE	use <Filename> to define annotation types.\n\
        \n\
     Clustering options:\n\
	-c FILE		Clusters mapping locations in FILE.\n\
	-cw X		Adds X to mapping position for clustering purposes.\n\
	-gff FILE	Clusters in gff format.\n\
	-bed FILE	Clusters in bedgraph format.\n\
	-description X	Adds X as a description to bedgraph files.\n\
     Other:\n\
	-uniq		merge identical input sequences. The sequence name will be 'sequence_count'.\n\
        -w, -write	create suffix array files.\n\
	-threads X	number of threads used (default 16)\n\
	-hash_size X	specify the number of query sequences stored in memory.\n\
	-expression-char	specify the character used for defining tag expression.\n\
        -q, -quiet	quite mode\n\
";
	
	param = malloc(sizeof(struct parameters));
	param->mode = 0;
	param->mismatches = 0;
	param->quiet_flag  = 0;
	param->suffixfile = 0;
	param->count_file = 0;
	param->count_matches = 0;
	param->unmapped_fasta = 0;
	param->mapped_fasta = 0;
	param->multi_mapped_fasta = 0;
	param->upper_limit = 10;
	param->fasta_file = 0;
	param->infiles = 0;
	param->infile = 0;
	param->num_types =0;
	param->format = 0;
	param->outfile = 0;
	param->outfile_tmp = 0;
	param->write_suffix = 0;
	param->split_counts = 0;
	param->time = 0;
	param->solid = 0;
	param->stats = 0;
	param->ann = 0;
	param->max_annotation_overlap = 2;
	param->r_output = 0;
	param->type_file = 0;
	param->types = 0;
	param->fragment = 0;
	param->short_seq = 0;
	param->query_num_limit = 1000000;
	param->minlen_cutoff = 0;
	param->maxlen_cutoff = MAX_LINE;
	param->uniq = 0;
	param->fast = 0;
	param->num_threads = 16;
	param->chop_at = 500;
	param->ex_char = "_";
	param->sort = 1;
	param->cluster = -1;
	param->gff = 0;
	param->bed = 0;
	param->mapped_expression = 0.0;
	param->hierarchical = 0;
	param->input_clustering = 0;
	param->description = 0;
	
	//opterr = 0;
	
	while (1){	
		static struct option long_options[] ={
			{"indel",0,0,OPT_INDEL},
   			{"unmapped",required_argument,0,OPT_UNMAPPED},
			{"mapped",required_argument,0,OPT_MAPPED},
	 		{"multi",required_argument,0,OPT_MULTI},
   			{"split",required_argument,0,OPT_SPLIT},
			{"uniq",0,0,OPT_UNIQ},
			{"time",0,0,'t'},
			{"fast",0,0,OPT_FAST},
   			{"solid",0,0,OPT_SOLID},
  			{"stats",required_argument,0,OPT_STATS},
			{"max_no_ann",required_argument,0,OPT_MAX_NO_ANN},
			{"types",required_argument,0,OPT_TYPES},
			{"ann",required_argument,0,OPT_ANN},
			{"frag",required_argument,0,OPT_FRAGMENT},
			{"short",0,0,OPT_SHORT},
			{"nosort",0,0,OPT_SORT},
   			{"cluster-window",required_argument,0,OPT_CLUSTER},
			{"cw",required_argument,0,OPT_CLUSTER},
      			{"gff",required_argument,0,OPT_GFF},
	 		{"bed",required_argument,0,OPT_BED},
 			{"hash_size",required_argument,0,OPT_QUERY_NUM_LIMIT},
    			{"min-len",required_argument,0,OPT_MINLENCUTOFF},
			{"max-len",required_argument,0,OPT_MAXLENCUTOFF},
			{"min_len",required_argument,0,OPT_MINLENCUTOFF},
   			{"max_len",required_argument,0,OPT_MAXLENCUTOFF},
			{"expression-char",required_argument,0,OPT_EX_CHAR},
   			{"expression_char",required_argument,0,OPT_EX_CHAR},
      			{"threads",required_argument,0,OPT_NUM_THREADS},
	 		{"chop_at",required_argument,0,OPT_CHOP_AT},
			{"chop-at",required_argument,0,OPT_CHOP_AT},
			{"hierarchical",0,0,OPT_HIERARCHICAL},
   			{"description",required_argument,0,OPT_DESCRIPTION},
			{"help",0,0,'h'},
			{"format",required_argument,0,'f'},
			{"quiet",0,0,'q'},
  			{"count",required_argument,0,OPT_COUNT},
   			{"write",required_argument,0,'w'},
			{0, 0, 0, 0}
		};
		
		int option_index = 0;
		c = getopt_long_only (argc, argv,"em:f:u:c:p:o:qhwr:t",long_options, &option_index);
		
		if (c == -1){
			break;
		}
		
		switch(c) {
			case 0:
				
				break;
			case OPT_HIERARCHICAL:
				param->hierarchical = 1;
				break;
			case OPT_UNMAPPED:
				if(optarg[0] == '-'){
					fprintf(stderr,"option -unmapped requires an argument\n");
					exit(-1);
				}
				param->unmapped_fasta = optarg;
				break;
			case OPT_MAPPED:
				if(optarg[0] == '-'){
					fprintf(stderr,"option -mapped requires an argument\n");
					exit(-1);
				}
				param->mapped_fasta = optarg;
				break;
			case OPT_MULTI:
				if(optarg[0] == '-'){
					fprintf(stderr,"option -multi requires an argument\n");
					exit(-1);
				}
				param->multi_mapped_fasta = optarg;
				break;
			case OPT_SOLID:
				param->solid = 1;
				break;
			case OPT_NUM_THREADS:
				param->num_threads = atoi(optarg);
				if(optarg[0] == '-'){
					fprintf(stderr,"option -threads requires an argument\n");
					exit(-1);
				}
				break;
			case OPT_MAX_NO_ANN:
				if(optarg[0] == '-'){
					fprintf(stderr,"option -max_no_ann requires an argument\n");
					exit(-1);
				}
				param->max_annotation_overlap = atoi(optarg);
				break;
			case OPT_QUERY_NUM_LIMIT:
				if(optarg[0] == '-'){
					fprintf(stderr,"option -max_no_ann requires an argument\n");
					exit(-1);
				}
				param->query_num_limit = atoi(optarg);
				break;
			case OPT_SPLIT:
				if(optarg[0] == '-'){
					fprintf(stderr,"option -split requires an argument\n");
					exit(-1);
				}
				param->split_counts = 1;
				param->count_file = optarg;
				break;
			case OPT_TYPES:
				if(optarg[0] == '-'){
					fprintf(stderr,"option -types requires an argument\n");
					exit(-1);
				}
				param->type_file = optarg;
				break;
			case OPT_EX_CHAR:
				if(optarg[0] == '-'){
					fprintf(stderr,"option -expression-char requires an argument\n");
					exit(-1);
				}
				param->ex_char = optarg;
				break;
			case OPT_INDEL:
				param->mode |= 4;
				break;
			case OPT_UNIQ:
				param->uniq = 1;
				break;
			case 't':
				param->time = 1;
				break;
				
			case OPT_SORT:
				param->sort = 0;
				break;
			case OPT_CLUSTER:
				//param->sort = 1;
				if(optarg[0] == '-'){
					fprintf(stderr,"option -cluster requires an argument\n");
					exit(-1);
				}
				param->cluster = atoi(optarg);
				break;
			case OPT_GFF:
				//param->sort = 1;
				if(param->cluster == -1){
					param->cluster = 0;
				}
				if(optarg[0] == '-'){
					fprintf(stderr,"option -gff requires an argument\n");
					exit(-1);
				}
				param->gff =optarg;
				break;
			case OPT_DESCRIPTION:
				if(optarg[0] == '-'){
					fprintf(stderr,"option -description requires an argument\n");
					exit(-1);
				}
				param->description = optarg;
				break;
			case OPT_BED:
				//param->sort = 1;
				if(param->cluster == -1){
					param->cluster = 0;
				}
				if(optarg[0] == '-'){
					fprintf(stderr,"option -bed requires an argument\n");
					exit(-1);
				}
				param->bed =optarg;
				break;	
			case OPT_ANN:
				if(optarg[0] == '-'){
					fprintf(stderr,"option -ann requires an argument\n");
					exit(-1);
				}
				param->ann = optarg;
				break;
			case OPT_STATS:
				if(optarg[0] == '-'){
					fprintf(stderr,"option -stats requires an argument\n");
					exit(-1);
				}
				param->stats = optarg;
				break;
			case OPT_CHOP_AT:
				if(optarg[0] == '-'){
					fprintf(stderr,"option -chop-at requires an argument\n");
					exit(-1);
				}
				param->chop_at = atoi(optarg);
				break;
			case OPT_FRAGMENT:
				if(optarg[0] == '-'){
					fprintf(stderr,"option -frag requires an argument\n");
					exit(-1);
				}
				param->fragment = atoi(optarg);
				break;
			case OPT_SHORT:
				param->short_seq = 1;
				break;
			case OPT_MINLENCUTOFF:
				if(optarg[0] == '-'){
					fprintf(stderr,"option -min-len requires an argument\n");
					exit(-1);
				}				
				param->minlen_cutoff = atoi(optarg);
				break;
			case OPT_MAXLENCUTOFF:
				if(optarg[0] == '-'){
					fprintf(stderr,"option -min-len requires an argument\n");
					exit(-1);
				}				
				param->maxlen_cutoff = atoi(optarg);
				break;
			case OPT_FAST:
				param->fast = 1;
				break;
			case 'e':
				param->mode |= 1;
				break;
			case 'm':
				if(optarg[0] == '-'){
					fprintf(stderr,"option -m requires an argument\n");
					exit(-1);
				}
				param->mode |= 2;
				param->mismatches = atoi(optarg);
				break;
			case 'f':
				if(optarg[0] == '-'){
					fprintf(stderr,"option -f requires an argument\n");
					exit(-1);
				}
				format = optarg;
				break;
			case 'u':
				if(optarg[0] == '-'){
					fprintf(stderr,"option -u requires an argument\n");
					exit(-1);
				}
				param->upper_limit = atoi(optarg);
				break;
			case OPT_COUNT:
				if(optarg[0] == '-'){
					fprintf(stderr,"option -c requires an argument\n");
					exit(-1);
				}
				param->count_matches = 1;
				param->count_file = optarg;
				break;
			case 'c':
				if(optarg[0] == '-'){
					fprintf(stderr,"option -c requires an argument\n");
					exit(-1);
				}
				if(param->outfile){
					fprintf(stderr,"option -c and -o cannot be used simultaneously\n");
					exit(-1);
				}
				
				//param->sort = 1;
				if(param->cluster == -1){
					param->cluster = 0;
				}
				param->outfile = optarg;
				param->input_clustering = 1;
				break;
			case 'p':
				if(optarg[0] == '-'){
					fprintf(stderr,"option -p requires an argument\n");
					exit(-1);
				}
				param->fasta_file = optarg;
				break;
			case 'o':
				if(optarg[0] == '-'){
					fprintf(stderr,"option -o requires an argument\n");
					exit(-1);
				}
				if(param->outfile){
					fprintf(stderr,"option -c and -o cannot be used simultaneously\n");
					exit(-1);
				}
				param->outfile = optarg;
				break;
			case 'q':
				param->quiet_flag = 1;
				break;
			case 'w':
				param->write_suffix = 1;
				break;
			case 'r':
				param->r_output = 1;
				break;
			case 'h':
				help = 1;
				break;
			case '?':
				exit(1);
				break;
			default:
				fprintf(stderr,"default\n\n\n\n");
				abort ();
		}
	}
	
	if(!param->quiet_flag){
		fprintf(stderr,"%s%s",version,usage);
	}
	if(help){
		fprintf(stderr,"%s\n",help_text);
		free(param);
		exit(0);
	}
	
	if(param->minlen_cutoff  > param->maxlen_cutoff){
		fprintf(stderr,"Error: -min-len has to be smaller than -max-len.\n");
		free(param);
		exit(-1);
	}
	
	
	if(!param->description){
		param->description = param->fasta_file;
	} 
	
	if(param->num_threads <1){
		param->num_threads = 1;
	}
	
	if(!param->mode){
		param->mode = 1;
	}

	if(param->query_num_limit > 70000000){
		fprintf(stderr,"ERROR: the maximum hash_size is 70 million.\n");
		free(param);
		exit(-1);
	}
	
	if(param->query_num_limit < 1000){
		param->query_num_limit = 1000;
		fprintf(stderr,"WARNING: the minimum hash_size is 1000.\n");
	}
	
	if(param->mismatches > 5){
		fprintf(stderr,"ERROR: no more than 5 mismatches allowed.\n");
		free(param);
		exit(-1);
	}
	
	if(param->split_counts && param->stats){
		fprintf(stderr,"ERROR: use -split OR -ann not both.\n");
		free(param);
		exit(-1);
	}
	
	if(param->ann && param->stats){
		fprintf(stderr,"ERROR: use -ann OR -stats not both.\n");
		free(param);
		exit(-1);
	}
	
	if(!param->outfile){
		if(param->input_clustering){
			fprintf(stderr,"Error: you need to specify the file to cluster (-c 'file')\n");
			free(param);
			exit(-1);
		}
		if(!param->write_suffix){
			if(param->upper_limit > 0){
				fprintf(stderr,"Error: you need to specify the output file (-o 'file')\n");
				free(param);
				exit(-1);
			}
		}
	}
	
	if(param->outfile){
		outlen = strlen(param->outfile);
		param->outfile_tmp = malloc(sizeof(char)*(outlen+5));
		for (i= 0;i < outlen;i++){
			param->outfile_tmp[i] = param->outfile[i];
		}
		param->outfile_tmp[outlen+0] = '.';
		param->outfile_tmp[outlen+1] = 't';
		param->outfile_tmp[outlen+2] = 'm';
		param->outfile_tmp[outlen+3] = 'p';
		param->outfile_tmp[outlen+4] = 0;
	}
	
	//fprintf(stderr,"MODE:%d\n",param->light);
	//if(param->quiet_flag){
	//	fclose(stderr);
	//}
	
	if(param->input_clustering){
		outlen = strlen(param->outfile);
		param->fasta_file = malloc(sizeof(char)*(outlen+9));		
		for (i= 0;i < outlen;i++){
			param->fasta_file[i] = param->outfile[i];
		}
		
		param->fasta_file[outlen+0] = '.';
		param->fasta_file[outlen+1] = 'c';
		param->fasta_file[outlen+2] = 'l';
		param->fasta_file[outlen+3] = 'u';
		param->fasta_file[outlen+4] = 's';
		param->fasta_file[outlen+5] = 't';
		param->fasta_file[outlen+6] = 'e';
		param->fasta_file[outlen+7] = 'r';
		param->fasta_file[outlen+8] = 0;
		return param;
	}
	
	if(param->count_matches && param->split_counts){
		fprintf(stderr,"ERROR: use -c/-count OR -split not both.\n");
		free(param);
		exit(-1);
	}
	
	/*if(!format){
	//	fprintf(stderr,"Default output format\n");
		param->format = 0;
	}else if(strcmp(format,"blast8")==0){
	//	fprintf(stderr,"blast8 output format\n");
		param->format = 1;
	}else if(strcmp(format,"blast9")==0){
	//	fprintf(stderr,"blast9 output format\n");
		param->format = 2;
	}else if(strcmp(format,"gff")==0){
	//	fprintf(stderr,"gff output format\n");
		param->format = 3;
	}else{
		fprintf(stderr,"ERROR: unrecognized output format: %s\n",format);
		exit(0);
	}*/
	//fprintf(stderr,"%ud\n",param->upper_limit);
	//fprintf(stderr,"%ud\n",UINT_MAX);
	if(param->upper_limit > UINT_MAX){
		fprintf(stderr,"WARNING: only up to %d matches per query sequence can be reported\n",UINT_MAX);
		param->upper_limit = UINT_MAX;
	}
	//fprintf(stderr,"%d\n",param->mode);
	//exit(0);
	//fprintf(stderr,"%d	%d\n",argc,optind);
	param->infile = malloc(sizeof(char*)* (argc-optind));
	c = 0;
	while (optind < argc){
		param->infile[c] = argv[optind++];
		//fprintf(stderr,"%s\n",argv[optind++]);
		c++;
	}
	param->infiles = c;
	/*if(!param->repeat){
		if(param->infiles > 32 && param->stats && param->split_counts){
			fprintf(stderr,"Cannot support stats for more than 32 input databases....\n");
			free(param);
			exit(0);
		}
	}*/
	if(param->infiles){
		param->suffixfile = malloc(sizeof(char*)*param->infiles);
		param->solid_suffixfile = malloc(sizeof(char*)*param->infiles);
		for(c =0;c< param->infiles;c++){
			outlen = strlen(param->infile[c]);
			param->suffixfile[c] = malloc(sizeof(char)*(outlen+10));
			param->solid_suffixfile[c] = malloc(sizeof(char)*(outlen+16));
			for (i= 0;i < outlen;i++){
				param->suffixfile[c][i] = param->infile[c][i];
				param->solid_suffixfile[c][i] = param->infile[c][i];
			}
			
			param->solid_suffixfile[c][outlen+0] = '.';
			param->solid_suffixfile[c][outlen+1] = 's';
			param->solid_suffixfile[c][outlen+2] = 'o';
			param->solid_suffixfile[c][outlen+3] = 'l';
			param->solid_suffixfile[c][outlen+4] = 'i';
			param->solid_suffixfile[c][outlen+5] = 'd';
			param->solid_suffixfile[c][outlen+6] = '_';
			param->solid_suffixfile[c][outlen+7] = 'n';
			param->solid_suffixfile[c][outlen+8] = 'e';
			param->solid_suffixfile[c][outlen+9] = 'x';
			param->solid_suffixfile[c][outlen+10] = 'a';
			param->solid_suffixfile[c][outlen+11] = 'l';
			param->solid_suffixfile[c][outlen+12] = 'i';
			param->solid_suffixfile[c][outlen+13] = 'g';
			param->solid_suffixfile[c][outlen+14] = 'n';
			param->solid_suffixfile[c][outlen+15] = 0;
			
			param->suffixfile[c][outlen+0] = '.';
			param->suffixfile[c][outlen+1] = 'n';
			param->suffixfile[c][outlen+2] = 'e';
			param->suffixfile[c][outlen+3] = 'x';
			param->suffixfile[c][outlen+4] = 'a';
			param->suffixfile[c][outlen+5] = 'l';
			param->suffixfile[c][outlen+6] = 'i';
			param->suffixfile[c][outlen+7] = 'g';
			param->suffixfile[c][outlen+8] = 'n';
			param->suffixfile[c][outlen+9] = 0;
		}
	}else{
		fprintf(stderr,"ERROR: no database file found.....\n");
		exit(-1);
	}
	//test files:
	if(!param->write_suffix){
		if (!(file = fopen(param->outfile, "w" ))){
			fprintf(stderr,"Cannot open file '%s'\n", param->outfile);
			exit(-1);
		}else{
			fclose(file);
			unlink(param->outfile);
		}
		if ((file = fopen(param->outfile_tmp, "w")) == NULL){
			fprintf(stderr,"Cannot open file '%s'\n", param->outfile_tmp);
			exit(-1);
		}else{
			fclose(file);
			unlink(param->outfile_tmp);
			
		}
	}
	
	if(param->stats){
		if(strcmp(param->stats,"STDOUT")){
			if ((file = fopen(param->stats, "w")) == NULL){
				fprintf(stderr,"Cannot open file '%s'\n", param->stats);
				exit(-1);
			}else{
				fclose(file);
				unlink(param->stats);
			}
		}
	}
	
	if(param->gff){
		if((file = fopen(param->gff, "w")) == NULL){
			fprintf(stderr,"Cannot open file '%s'\n", param->gff);
			exit(-1);
		}else{
			fclose(file);
			unlink(param->gff);
		}
		
	}
	
	if(param->bed){
		if((file = fopen(param->bed, "w")) == NULL){
			fprintf(stderr,"Cannot open file '%s'\n", param->bed);
			exit(-1);
		}else{
			fclose(file);
			unlink(param->bed);
		}
	}
	
	if(param->unmapped_fasta){
		if((file = fopen(param->unmapped_fasta, "w")) == NULL){
			fprintf(stderr,"Cannot open file '%s'\n", param->unmapped_fasta);
			exit(-1);
		}else{
			fclose(file);
			unlink(param->unmapped_fasta);
		}
	}
	
	if(param->mapped_fasta){
		if((file = fopen(param->mapped_fasta, "w")) == NULL){
			fprintf(stderr,"Cannot open file '%s'\n", param->mapped_fasta);
			exit(-1);
		}else{
			fclose(file);
			unlink(param->mapped_fasta);
		}
	}
	
	if(param->multi_mapped_fasta){
		if((file = fopen(param->multi_mapped_fasta, "w")) == NULL){
			fprintf(stderr,"Cannot open file '%s'\n", param->multi_mapped_fasta);
			exit(-1);
		}else{
			
			fclose(file);
			unlink(param->multi_mapped_fasta);
		}
	}
	
	return param;
}

void param_sanity(struct parameters* param)
{
	int c;
	int outlen;
	int i;
	if(!param->mode){
		param->mode = 1;
	}
	
	if(param->mismatches > 3){
		fprintf(stderr,"No more than 3 mismatches allowed.\n");
		param->abort = 1;
		//free(param);
		//exit(0);
	}
	
	if(param->split_counts){
		if(param->unmapped_fasta){
			fprintf(stderr,"Error: '--split, --ann, --repeat' is not allowed in combination with '--unmapped'.\n");
			param->abort = 1;
		}
		if(param->mapped_fasta){
			fprintf(stderr,"Error: '--split, --ann, --repeat' is not allowed in combination with '--mapped'.\n");
			param->abort = 1;
		}
		if(param->multi_mapped_fasta){
			fprintf(stderr,"Error: '--split, --ann, --repeat' is not allowed in combination with '--multi'.\n");
			param->abort = 1;
		}
	}
	
	if(param->quiet_flag){
		fclose(stderr);
	}
	
	if(param->upper_limit > UINT_MAX){
		fprintf(stderr,"WARNING: Only up to %d matches per query sequence can be reported\n",UINT_MAX);
		param->upper_limit = UINT_MAX;
	}
	
	/*if(!param->types){
	 if(param->infiles > 32 && param->stats && param->split_counts){
	 fprintf(stderr,"Cannot support stats for more than 32 input databases....\n");
	 param->abort = 1;
	 }
	 }*/
	if(param->infiles && !param->abort){
		param->suffixfile = malloc(sizeof(char*)*param->infiles);
		assert(param->suffixfile != 0);
		param->solid_suffixfile = malloc(sizeof(char*)*param->infiles);
		assert(param->solid_suffixfile != 0);
		for(c =0;c < param->infiles;c++){
			outlen = strlen(param->infile[c]);
			param->suffixfile[c] = malloc(sizeof(char)*(outlen+10));
			assert(param->suffixfile[c] != 0);
			param->solid_suffixfile[c] = malloc(sizeof(char)*(outlen+16));
			assert(param->solid_suffixfile[c] != 0);
			for (i= 0;i < outlen;i++){
				param->suffixfile[c][i] = param->infile[c][i];
				param->solid_suffixfile[c][i] = param->infile[c][i];
			}
			param->solid_suffixfile[c][outlen+0] = '.';
			param->solid_suffixfile[c][outlen+1] = 's';
			param->solid_suffixfile[c][outlen+2] = 'o';
			param->solid_suffixfile[c][outlen+3] = 'l';
			param->solid_suffixfile[c][outlen+4] = 'i';
			param->solid_suffixfile[c][outlen+5] = 'd';
			param->solid_suffixfile[c][outlen+6] = '_';
			param->solid_suffixfile[c][outlen+7] = 'n';
			param->solid_suffixfile[c][outlen+8] = 'e';
			param->solid_suffixfile[c][outlen+9] = 'x';
			param->solid_suffixfile[c][outlen+10] = 'a';
			param->solid_suffixfile[c][outlen+11] = 'l';
			param->solid_suffixfile[c][outlen+12] = 'i';
			param->solid_suffixfile[c][outlen+13] = 'g';
			param->solid_suffixfile[c][outlen+14] = 'n';
			param->solid_suffixfile[c][outlen+15] = 0;
			
			param->suffixfile[c][outlen+0] = '.';
			param->suffixfile[c][outlen+1] = 'n';
			param->suffixfile[c][outlen+2] = 'e';
			param->suffixfile[c][outlen+3] = 'x';
			param->suffixfile[c][outlen+4] = 'a';
			param->suffixfile[c][outlen+5] = 'l';
			param->suffixfile[c][outlen+6] = 'i';
			param->suffixfile[c][outlen+7] = 'g';
			param->suffixfile[c][outlen+8] = 'n';
			param->suffixfile[c][outlen+9] = 0;
		}
	}else{
		fprintf(stderr,"No database file found.....\n");
		param->abort = 1;
	}
}

