.TH "nexalign" 1 "Nov 2008" "Nexalign 1.3.5" "Nexalign Manual"

.SH NAME
.TP 
nexalign - aligns sequences to databases

.SH SYNOPSIS
.B nexalign
.I [options]
-p 
.I query
.I database1
.I database2   
.I ...
-o 
.I output

.SH DESCRIPTION

Nexalign aligns sequence reads from a single 
.I query
file to multiple 
.I database
files and writes the alignments to 
.I output.

.PP
Nexalign is designed to work efficiently with millions of short reads generated by e.g. Roche's 454, Illumina's Solexa or ABI's SOliD sequencing technology. Nexalign reports all alignments with up to one insertion/deletion or with up to 4 mismatches. In addition to the alignments themselves, nexalign can output the mapped or unmapped tags in fasta format facilitating the construction of complex mapping pipelines.

.PP
The
.I query
file can be in either standard fasta or color-space fasta format (XXX.csfasta). All
.I database
files have to be in standard fasta format. If no 
.I query 
file is specified nexalign tries to read sequences from standard input. 

.PP
If sequence names in the 
.I query 
file are followed by   
.I '_X'
nexalign assumes that
.I 'X'
is the number of times the sequence was observed. Otherwise it assign a count of 1 to all sequences. This default behavior can be changed using the
.B -expression-char 
option.

.SH ALIGNMENT OPTIONS

.TP
.B -e 
Reports exact alignments.

.TP
.BI -m " <x>"
Reports alignments with up to
.I x
mismatches. 

.TP
.B -indel 
Reports alignments with one insertion or deletion. 

.PP 
.B Note:
Each alignment option turns off the other two alignments options. I.e. -m 
.I x 
will produce mismatched alignments but not exact alignments. To obtain both exact and mismatched alignments both options have to be used: " -e -m 1".

.TP
.B -solid 
Align SOliD color space sequences. 

.SH SEQUENCE SELECTION OPTIONS

.TP
.BI -min-len " <x>"
Align tags equal or longer than
.I 'X'
(default: 0).

.TP
.BI -max-len " <x>"
Align tags equal or shorter than
.I 'X'
(default: 500).

.TP
.BI -chop_at " <x>" 
Align only the first
.I <x>
bases of every tag. This is applied after the -min-len / -max-len checks.

.TP
.BI -frag " <x>"
Fragments the input tags into 
.I <x>
-mers. The sequence counts are divided up equally among the fragments. This option is applied after all of the above. 


.SH OUTPUT OPTIONS

.TP
.BI -o " <f>"
Writes alignments to file
.I <f>.

.TP
.BI -unmapped " <f>" 
Writes all unmapped sequences to file
.I <f>.

.TP 
.BI -mapped " <f>" 
Writes all mapped sequences to file
.I <f>.
A sequence is considered mapped if it aligns to less than 100 places in all input 
.I databases. 

.TP 
.BI -multi " <f>" 
Writes all ambiguously mapped sequences to file
.I <f>.
A sequence is considered to be ambiguously mapped if it aligns to more than 100 places in all input
.I databases. 

.TP
.BI -u " <x>"
Specifies the maximum number of alignments reported per
.I query
sequence. This option will not stop reporting alignments at a certain point but rather only report alignments for sequences that map less than 
.I <x>
times. Additionally, this option is used to modify the cutoffs for options
.B -mapped
and
.B -multi.
For example, "-u 1 -mapped 
.I <f>  
" will write alignments for sequences mapping only once and print the corresponding sequences to file
.I <f>.
Exception: Using "-u 0 -mapped 
.I <f>  
" will not result in an empty 
.I <f> 
as expected, but will instead print all mapping sequences to 
.I <f>.
.I Default:
100.

.SH REPORTING OPTIONS

.TP
.BI -stats " <f>"
Prints overall mapping statistics to file 
.I <f>. 
If the R statistical package is available pdf and jpeg files are also created. Setting <f> to STDOUT will print the statistics to the standard output.

.TP
.BI -count " <f>"
Prints a summary of overall mapping statistics for each sequence to file 
.I <f>.

.TP
.BI -split " <f>"
Prints a summary of mapping statistics for each sequence and each 
.I database
to file 
.I <f>.

.SH ANNOTATION OPTIONS
.TP 
.BI -ann " <f>"
Prints an overview of the percentage of sequences mapping to specific
.I databases
to file 
.I <f>.
If the R statistical package is available pdf and jpeg files are also created. Setting <f> to STDOUT will print the statistics to the standard output.


.TP
.BI -max_no_ann " <x>"
This option modifies the behavior of "-ann". 
.I <x> 
specifies the maximum number of annotation databases a sequence can map to before being considered ambiguously annotated.
.I Default:
2.

.TP 
.BI -types " <f>"
When working with large annotation databases it is desirable to split them up into several smaller fasta files due to potential memory limitations. For nexalign all the database files belonging to the same annotation category should contain a TAG in the filename in the format "_TAG".  The file 
.I <f> 
contains a list of all TAGs telling nexalign which annotation files belong together.

.SH CLUSTERING OPTIONS

.TP
.BI -c " <f>"  
Clusters mapping locations in file 
.I <f> 
into tag clusters. The file needs to be in standard nexalign output format.  

.TP
.BI -cluster-window " <x>" 
Adds a window of 
.I <x>
to each mapping position for clustering purposes. 

.TP
.BI -gff " <f>"
Creates a gff file containing the clusters.

.TP
.BI -bed " <f>"
Reformats clusters into bedgraph formatted files for both the negative and positive strand. 

.TP
.BI -description " <x>"
Add a UCSC viewable description to the bedgraph files. 

.PP 
.B Note:
Using options 
.B -gff 
or 
.B -bed
will cause nexalign to automatically cluster the mapping output.


.SH OTHER OPTIONS

.TP
.B -h
Prints a brief help. 

.TP
.B -q
Quiet mode - nothing is written to STDERR. 

.TP
.B -w
Creates indices for each 
.I database
file. These indices are written to the same directory as the
.I database
files. To create indices for SOliD data use '-w' in combination with '-solid' (see below). 

.TP
.B -time
Reports the running time of nexalign.

.TP
.BI -threads " <x>"
Number of threads used (default: 16).

.TP
.BI -expression-char " <c>"
Use 
.I <c>
instead of underscore to indicate the counts of tags in the input. 

.TP
.B -uniq
Merges identical input sequences into single query sequences. The names of the individual sequences are lost. Fasta files produced by nexalign will contain the sequence itself as the name followed by 
.I '_X' 
where 
.I X
reflects how often the sequence was observed in the input. 

.PP
.B Note:
If the hash_size (see below) is smaller that the number of sequences in the 
.I query
file, nexalign will only merge identical sequences within smaller chunks of the 
.I query.
In this case it is recommended to either increase the hash_size or convert the 
.I query
file into a file including the
.I '_X'
naming convention.


.TP
.BI -hash_size " <x>"
Defines the number of unique query sequences nexalign can hold in memory. If smaller than the number of input sequences 
.I <y>
, all databases have to be searched 
.I <y> / <x> 
times. On the other hand, defining 
.I <x> 
much larger than 
.I <y> 
increases execution time since nexalign always allocates a datastructure of size
.I <x>. 
.I Default: 
10000000.


.SH EXAMPLES

.SS Hierarchical Mapping:

nexalign -e -p query.fa /hg18/*.fa -o exact_mappings.csv -unmapped STDOUT | nexalign -m 1 /hg18/*.fa -o 1_mismatch_mappings.csv -unmapped STDOUT | nexalign -m 2 /hg18/*.fa -o 2_mismatch_mappings.csv

.PP
Maps query sequences to the human genome exactly, takes the remaining unmapped sequences and maps them first with 1, then with 2 mismatches.

.SS Annotation:

nexalign -e -p query.fa exon.fa intron.fa promoter.fa -ann annotation.csv -u 0 -max_no_ann 1

.PP
Displays the proportion of query sequences mapping to exons, introns, promoters. Since max_no_ann is set to one all sequences mapping to more than one category will be displayed as ambiguously annotated. 

.SS Nexalign "Queries":

nexalign -p query.fa ribosomal.fa -unmapped STDOUT | nexalign miRNA_hairpin.fa -mapped STDOUT | nexalign /hg18/*.fa -u 1 -mapped map.fa > mappings.csv

.PP 
Extracts all sequences from "query.fa" that do not map to ribosomal sequences, but map uniquely to miRNA hairpin sequences. 

.SH COPYRIGHT

.nf
Copyright (C) 2007-2008 Timo Lassmann
Freely distributed under the GNU General Public License (GPL).
.fi
See the file COPYING in your distribution for details on redistribution
conditions.

.SH AUTHOR 

.nf
Timo Lassmann
Researcher LSA Bioinformatics Team
Omics Science Center (OSC),
Riken Yokohama Institute
1-7-22 Suehiro-chu, Tsurumi-ku, Yokohama
230-0045 Kanagawa, Japan 

email: timolassmann@gmail.com
http://genome.gsc.riken.jp/osc/english/members/Timo_Lassmann.html
.fi


