/*
	tag_searching.h
	
	Released under GPL - see the 'COPYING' file   
	
	Copyright (C) 2006 Timo Lassmann <timolassmann@gmail.com>
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
	Please send bug reports, comments etc. to:
	timolassmann@gmail.com
*/

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <limits.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include<fcntl.h>
#include <sys/wait.h>
#include <pthread.h>

#include<assert.h>

#define SEEK_START 0
#define SEEK_END 2

#define NUM_THREADS thread


#define VERSION "Nexalign version 1.3.5"
#define OUTPUT_VERSION "Nexalign output version 2"

#define KEY_LEN 16
#define MAX_LINE 500
#define SUFFIX_VERSION 3


struct parameters{
	char** infile;
	char** suffixfile;
	char** solid_suffixfile;
	char** types;
	char** type_colors;
	char* fasta_file;
	
	char* count_file;
	char* unmapped_fasta;
	char* mapped_fasta;
	char* multi_mapped_fasta;
	char* outfile;
	char* outfile_tmp;
	char* stats;
	
	char* type_file;
	
	char* ann;
	char* ex_char;
	char* gff;
	char* bed;
	char* description;
	
	double mapped_expression;
	
	int r_output; 
	
	int fast;
	int fragment;
	int num_types;	
	int abort;
	int max_annotation_overlap;
	int solid;
	int time;
	int uniq;
	int format;
	int mode;
	int mismatches;
	int quiet_flag;
	unsigned int upper_limit;
	int infiles;
	int write_suffix;
	int split_counts;
	int count_matches;
	int short_seq;
	int query_num_limit;
	int minlen_cutoff;
	int maxlen_cutoff;
	int num_threads;
 	int chop_at;
	int sort;
	int cluster;
	int hierarchical;
	int input_clustering;
	
	
};

struct fasta{
	char** sn;
	char* string;
	int* mer_hash;
	int* boost;
	int* s_index;
	int* suffix;
	int numseq;
	int max_len;
	int string_len;
};

struct query{
	struct query* next;
	char* seq;
	char* sn;
	unsigned int stats[8];
	double count;
	unsigned int len;
	unsigned int sn_len;
};

struct query_table{
	struct query** hash; 
	struct query** park;
	struct p_node* tree;
	double** stats;
	char* string;
	double total_expression;
	int numseq;
	int size;
	int tree_size; 
};

struct suffix_pos{
	int s_p;
	int e_p;
	int s_m;
	int e_m;
	int sum;
	int mis;
	int indel;
};

struct p_node{
	int key;
	double count;
	struct p_node* left;
	struct p_node* right;
};


#ifdef thread
struct thread_data
{
	struct query_table* q;
	struct fasta* t;
	struct parameters* param;
	int start;
	int end;
};

void* pattern_matching_thread(void *threadarg);
void* pattern_matching_quick_thread(void *threadarg);
void* pattern_matching_short_thread(void *threadarg);
void* pattern_matching_solid_thread(void *threadarg);
void* pattern_matching_solid_quick_thread(void *threadarg);
#endif

void create_suffix(struct parameters* param);

int base_space_errors(const char* p,const char* t,int len);
int base_space_errors_rev(const char* p,const char* t,int len);

int test_solid(struct query_table* qt);
//void pattern_matching_solid(struct fasta* q,const struct fasta* t,const struct parameters* param);

void mapping(struct query_table* qt, struct parameters* param);
void mapping_solid(struct query_table* qt, struct parameters* param);

void pattern_matching(struct query_table* qt,const struct fasta* t,const struct parameters* param);
void pattern_matching_quick(struct query_table* qt,const struct fasta* t,const struct parameters* param);
void pattern_matching_short(struct query_table* qt,const struct fasta* t,const struct parameters* param);
void pattern_matching_solid(struct query_table* qt,const struct fasta* t,const struct parameters* param);
void pattern_matching_solid_quick(struct query_table* qt,const struct fasta* t,const struct parameters* param);

long int t_long_strncmp(const char *s,const char *t,const int len);
unsigned long int rev(unsigned long int x);



char* get_time(char* time_s);
void print_date();
struct parameters* interface(struct parameters* param,int argc,char **argv);

struct fasta* get_fasta(struct fasta* p,char *infile,int solid);
char* get_input_into_string(char* string,char* infile);

struct fasta* read_fasta(struct fasta* aln);

int get_suffix_len(char* string);
int* read_suffix(int* suffix,char* infile,int t_len);

//void pattern_matching(struct fasta* q,const struct fasta* t,const struct parameters* param);


int binsearch_down(const char*p,const char* text,const int* suffix,int len,int h);
int binsearch_up(const char*p,const char* text,const int* suffix,int len,int h);
int binsearch_up2(const char*p,const char* text,const int* suffix,const int len,int h);
void binsearch_up3(struct suffix_pos* sp,const char*p,const char* text,const int* suffix,const int len,int h);

int binsearch_down4(const char*p,const char* text,const int* suffix,const int len,int l,int h);
int binsearch_up4(const char*p,const char* text,const int* suffix,const int len,int l,int h);



void reverse_complement(const char* p,char* rc,int len);

//char* reverse_complement(const char* p,char* rc,int len);
void reverse(const char* p,char* rc,const int len);

void pop_stats(struct query_table* qt,struct parameters* param);

void print_counts(struct query_table* qt,struct parameters* param,int db);
void clear_counts(struct query_table* qt);
void clear_6_counts(struct query_table* qt);

void print_stats(struct query_table* qt,struct parameters* param);
void report_stats(struct query_table* qt,struct parameters* param);
//void print_summary(struct fasta* q,struct parameters* param);
void print_unmapped_fasta(struct query_table* qt,char* outfile);
void print_mapped_fasta(struct query_table* qt,char* outfile,unsigned int ulimit);
void print_multi_mapped_fasta(struct query_table* qt,char* outfile,const unsigned int ulimit);

//void print_split_summary(struct fasta* q,struct parameters* param);
void print_ann(struct query_table* qt,struct parameters* param);

void pop_ann_tree(struct query_table* qt,int max_no_ann_overlap);

struct fasta* alloc_target(struct fasta* p,char** infiles,int num_infiles);
struct fasta* alloc_aln(struct fasta* aln,int size);

void free_names(struct fasta*aln);
void free_aln(struct fasta*aln);
void free_param(struct parameters* param);

int arbins(const int* x, const int size, const int y);
char* shorten_name(char* p);

int is_substr(const char* pattern,const char*text);
int byg_end(const char* pattern,const char*text);

FILE *foutfile;

void write_bin_aln(struct fasta* p,const char* file);
struct fasta* read_bin_aln(struct fasta* p,const char* file);

int pop(int x);

struct p_node* insert(struct p_node *n,int key, double count, int* size);
void print_tree(struct p_node* n,char** databases,double total,FILE *fout);
void get_ann_score(struct p_node* n,double* out,int max_ann_no);
int collect_pie_data(struct p_node* n,char** databases,char** colors,double* cat_expression,char** cat_name,char** cat_color,int cur,double total_expression);

void free_p_tree(struct p_node* n);


//struct fasta* determine_class(struct fasta* query,struct parameters* param, int db);
void set_class(struct query_table* qt,struct parameters* param, int db);

struct parameters* read_list(struct parameters* param,char* string);

void read_types_list(struct parameters* param);

struct query_table* init_query_hash(struct query_table* qt,struct parameters* param);
void free_query_hash(struct query_table* qt,struct parameters* param);

void init_r();
void insert_hash_with_name(struct query_table* qt,struct parameters* param, char *seq,char* name, int len, int sn_len, double count);
void insert_hash(struct query_table* qt,struct parameters* param, char *text,int len, double count);
void clear_hash_table(struct query_table* qt);
void print_hash(struct query_table* qt);

struct query_table* read_query_fasta_file_with_name(struct query_table* qt,struct parameters* param);
struct query_table* read_query_fasta_file(struct query_table* qt,struct parameters* param);
struct query_table* read_query_fasta_file_frag(struct query_table* qt,struct parameters* param,int frag);

void print_ann_score(struct query_table* qt,int max_no_ann);

void r_ann_pie(struct query_table* qt,struct parameters* param);
void r_stats_pie(struct query_table* qt,struct parameters* param);

void check_sequences(struct query_table* qt,struct parameters* param);

void mer_hash(struct fasta* target,int len);
unsigned int tag_index(const char* tag);

void simple_sort_hash(struct query_table* qt,int quiet);
struct query* lookup_hash(struct query_table* qt,char *text,int len);
struct query* search_hash(struct query_table* qt,struct query* q,char *seq);
void clean_mapping_output(struct query_table* qt,struct parameters* param);
void clean_name_mapping_output(struct query_table* qt,struct parameters* param);
void copy_mapping_output(struct query_table* qt,struct parameters* param);


void set_hash_size(struct parameters* param);


struct query* lookup_name_hash(struct query_table* qt,char *name,int len);
void insert_hash_name(struct query_table* qt,struct parameters* param, char *seq,char* name, int len, int sn_len, double count);

int test_tag_index(const char* tag,const int* mer_hash);
unsigned int tag_index2(const char* tag);
unsigned int tag_index_solid(const char* tag);

void reverse_complement2(const char* p,char* rc,int len);

void get_mapped_expression(struct parameters* param);
void sort_db(struct parameters* param);
void cluster_mapping(struct parameters* param);



